#line 353 "./lpsrc/flx_faio.pak"
#ifndef __ASYNCIO__
#define __ASYNCIO__
#include <flx_faio_config.hpp>

// this file acts as go-between for my asynchronous interface for felix
// programmes.

// portable. Not that asynchronous.

#include "demux_demuxer.hpp"        // sel_param, demuxer base
#include "faio_drv.hpp"

namespace flx { namespace faio {

class FAIO_EXTERN flx_driver_request_base {
public:
    virtual ~flx_driver_request_base() {}       // so destructors work
    // returns finished flag (async may fail or immediately finish)
    virtual bool start_async_op(demux::demuxer& demux, flx_drv* drv, void* f) = 0;
};

// not actually asynchronous as such since the threads are woken synchronously
// definitely portable. look out for this shared structure when flx threads
// are run pre-emptively. start_async_op will need to be serialised.
class FAIO_EXTERN async_copipe : public flx_driver_request_base {
public:
    enum { WINDWARD, LEEWARD, NUM_CHANNELS };

    virtual bool start_async_op(demux::demuxer& demux, flx_drv* drv, void* f);
    void connect(void* f, demux::sel_param* pb, bool reading, int channel);
    void close_channel(int which);      // WINDWARD or LEEWARD
    void disconnect();                  // will be used by close

    static async_copipe* create_copipe() { return new async_copipe; }
    bool debug;
    void set_debug(bool);

private:
    enum { READER, WRITER, NUM_CNXNS };

    // if there's a thread here, it's sleeping
    void*   thread[NUM_CNXNS];
    demux::sel_param*  pb[NUM_CNXNS];
    int         num_users;

    bool        channel_open[NUM_CHANNELS];
    int         current_channel;
    void wake_thread(int n, flx_drv* drv);
    void wake_all_threads(flx_drv* drv);

    async_copipe();                     // to control alloc/dealloc
};

class FAIO_EXTERN copipe_endpt {
    int             read_channel;       // the channel this endpt reads on

    copipe_endpt(async_copipe* p, int rchan) :
      read_channel(rchan), pipe(p), debug(false)
    {}

public:
    async_copipe*   pipe;               // endpt's copy (flx accessible)

    // 0 = no reads, 1 = no writes, 2 = no nothing (! = close, however)
    void shutdown(int how);
    int get_channel(bool reading);
    bool debug;
    void set_debug(bool d) { debug = d; }

    ~copipe_endpt() { pipe->disconnect(); }
    static void pipe_pair(copipe_endpt* pair[2]);
};

// sleeping
class FAIO_EXTERN sleep_request
  : public flx_driver_request_base, public demux::sleep_task
{
  thread_wakeup fw;
  double      delta;
public:
  sleep_request() {}        // flx linkage

  sleep_request(double d) : delta(d) {}

  // from driver request
  virtual bool start_async_op(demux::demuxer& demux, flx_drv* drv, void* f);

  // from sleep_task
  virtual void fire();
};

}} // namespace faio, flx
#endif  // __ASYNCIO__

