#line 870 "./lpsrc/flx_faio.pak"
#ifndef __FAIO_POSIXIO__
#define __FAIO_POSIXIO__
#include <flx_faio_config.hpp>

#include "faio_asyncio.hpp"
#include "faio_drv.hpp"

// we don't need to piggyback much data at all. for now just the demuxer,
// so that we can be woken up, and the buffer info (this replaces the
// felix "socket" thread type, which was ugly.

#include "demux_posix_demuxer.hpp"

// a new sort of demuxer/event source: file io completions
// haven't given up on using the socket style demuxers yet.
#include "demux_pfileio.hpp"

#include "demux_timer_queue.hpp"

namespace flx { namespace faio {

class FAIO_EXTERN socketio_wakeup : public demux::socket_wakeup {
public:
  demux::sel_param   pb;     // in: what you want, out: what you get
  thread_wakeup fw;

  int       sio_flags;  // either one of PDEMUX_{READ|WRITE}

  virtual void wakeup(demux::posix_demuxer& demux);
};

// this can handle most unix style io, that is, read & write on sockets,
// files & pipes. NICE. the fact that the socket is now in here may mean
// I can get rid of the epoll hack
// Not sure if this can be used for file fds.
class FAIO_EXTERN socketio_request : public flx_driver_request_base {
public:
    socketio_wakeup sv;

    socketio_request() {}       // Lord Felix demands it. Like STL.

    socketio_request(int s, char* buf, long len, bool r);
    virtual bool start_async_op(demux::demuxer& demux, flx_drv* drv,  void* f);
};

class FAIO_EXTERN connect_request
  : public flx_driver_request_base, public demux::connect_control_block {
public:
  thread_wakeup fw;

  connect_request() {}      // flx linkage

  connect_request(const char* addr, int port) { addy = addr; p = port; s=-1; }
  virtual bool start_async_op(demux::demuxer& demux, flx_drv* drv,  void* f);
  virtual void wakeup(demux::posix_demuxer& demux);
};

class FAIO_EXTERN accept_request
  : public flx_driver_request_base, public demux::accept_control_block {
public:
  // we sometimes know that there'll be several connections to accept.
  // this'll need a different wakeup - and a different interface between
  // event source & wakeups
  thread_wakeup fw;

  accept_request() {} // flx linkage

  // eeh, give that a better name
  accept_request(int listener) { s = listener; }

  // from flx_driver_request_base
  virtual bool start_async_op(demux::demuxer& demux, flx_drv* drv, void* f);

  // from accept_control_block
  virtual void wakeup(demux::posix_demuxer& demux);
};


// separate pthread file io
// hum. multiple inheritance
class FAIO_EXTERN flxfileio_request
    : public flx_driver_request_base, public demux::fileio_request
{
    thread_wakeup   fw;
public:
    flxfileio_request();           // flx linkage
    ~flxfileio_request();          // flx linkage

    flxfileio_request(int f, char* buf, long len, long off, bool rd)
        : fileio_request(f, buf, len, off, rd) {}

    // from driver request
    virtual bool start_async_op(demux::demuxer& demux, flx_drv* drv, void* f);

    // from async io thread
    virtual void finished();
};

}}
#endif
