#line 1201 "./lpsrc/flx_faio.pak"
#ifndef __DWINIO__
#define __DWINIO__
#include <flx_faio_config.hpp>

// visual studio is quite sensitve about how you do these includes.
// THIS is the way (WinSock2.h must include Windows.h).
#include <WinSock2.h>
#include <MSWSock.h>        // AcceptEx, TF_REUSE_SOCKET, etc

#include "faio_asyncio.hpp" // flx driver requests
#include "demux_overlapped.hpp"   // nicely wrapped async windows calls

namespace flx { namespace faio {

// interestingly, because in windows the async objects are associated
// with an IOCP before their use, we don't actually need a demuxer here
// at all. That's kind of nice. (actually iocp_associator uses it now)

// a flx driver request to the add socket s to the drivers iocp
// this is currently the only windows driver request that uses the demuxer.
class FAIO_EXTERN iocp_associator : public flx_driver_request_base {
  SOCKET  s;
public:
  // should have result & errcode
  iocp_associator() {}
  iocp_associator(SOCKET associatee) : s(associatee) {}

  virtual bool start_async_op(demux::demuxer& demux, flx_drv* drv, void* f);
};

// flx <-> c++ stuff for async io (well, it was)

// transition to new windows async control block
class FAIO_EXTERN waio_base : public flx_driver_request_base {
protected:
  thread_wakeup fw;
public:
  bool  success;          // eh?

  waio_base() : success(false) {}

  // actually wakes up thread
  virtual void iocp_op_finished( DWORD nbytes, ULONG_PTR udat,
    LPOVERLAPPED olp, int err);
};


// listener socket must be already associated with an IOCP
// in doing an AcceptEx, it might succeed immediately - do you still
// get the IOCP wakeup?
class FAIO_EXTERN wasync_accept
  : public waio_base, public demux::acceptex_control_block
{
public:
  wasync_accept() {}  // felix linkage demands it

  wasync_accept(SOCKET l, SOCKET a) { listener = l; acceptor = a; }

  virtual bool start_async_op(demux::demuxer& demux, flx_drv* drv, void* f);

  virtual void iocp_op_finished( DWORD nbytes, ULONG_PTR udat,
    LPOVERLAPPED olp, int err);
};

class FAIO_EXTERN connect_ex
  : public waio_base, public demux::connectex_control_block
{
public:

  connect_ex() {}     // flx linkage

  connect_ex(SOCKET soc, const char* addr, int port)
    { s = soc; addy = addr; p = port; }

  virtual bool start_async_op(demux::demuxer& demux, flx_drv* drv, void* f);

  virtual void iocp_op_finished( DWORD nbytes, ULONG_PTR udat,
    LPOVERLAPPED olp, int err);
};

// TransmitFile here (requires file handle)
class FAIO_EXTERN wasync_transmit_file
  : public waio_base, public demux::transmitfile_control_block
{
public:
  wasync_transmit_file()
    : transmitfile_control_block(INVALID_SOCKET, NULL) {}   // flx linkage

  wasync_transmit_file(SOCKET dst)      // for reuse of socket
    : transmitfile_control_block(dst) {}

  wasync_transmit_file(SOCKET dst, HANDLE src)  // actual transmitfile
    : transmitfile_control_block(dst, src) {}

  // from flx_request_base
  virtual bool start_async_op(demux::demuxer& demux, flx_drv* drv, void* f);

  virtual void iocp_op_finished(DWORD nbytes, ULONG_PTR udat,
    LPOVERLAPPED olp, int err);
};

// handles both WSASend & WSARecv
class FAIO_EXTERN wsa_socketio
  : public waio_base, public demux::wsasocketio_control_block
{
public:
  wsa_socketio()
    : wsasocketio_control_block(INVALID_SOCKET, NULL, false) {}

  wsa_socketio(SOCKET src, demux::sel_param* ppb, bool read)
    : wsasocketio_control_block(src, ppb, read) {}

  virtual bool start_async_op(demux::demuxer& demux, flx_drv* drv, void* f);

  virtual void iocp_op_finished( DWORD nbytes, ULONG_PTR udat,
    LPOVERLAPPED olp, int err);
};

// looks a bit like wsa_socketio (bad name, sends too)
class FAIO_EXTERN winfile_io
  : public waio_base, public demux::winfileio_control_block
{
public:
  winfile_io()      // flx linkage
    : winfileio_control_block(NULL, NULL, 0, false){}

  // offset?
  winfile_io(HANDLE f, void* buf, int len, bool read)
    : winfileio_control_block(f, buf, len, read) {}

  virtual bool start_async_op(demux::demuxer& demux, flx_drv* drv, void* f);

  virtual void iocp_op_finished( DWORD nbytes, ULONG_PTR udat,
    LPOVERLAPPED olp, int err);
};


}}
#endif  // __DWINIO__

