#line 2651 "./lpsrc/flx_rtl.pak"
#include "flx_async.hpp"
#include "pthread_sleep_queue.hpp"
#include "flx_rtl.hpp"
#include "demux_demuxer.hpp"
#include "faio_drv.hpp"

using namespace flx::rtl;
using namespace flx::demux;
using namespace flx::faio;
using namespace flx::pthread;


class async_hooker_impl : public async_hooker {
  virtual flx::faio::flx_drv *get_driver()=0;
  virtual flx::demux::demuxer *get_demuxer()=0;
public:
  bool handle_request(void *data,fthread_t *ss);
  ~async_hooker_impl();
};


#ifdef _WIN32
 #include "demux_iocp_demuxer.hpp"
 #include "faio_wdrv.hpp"
 typedef flx::demux::iocp_demuxer flx_demuxer;
#elif defined(HAVE_KQUEUE_DEMUXER)
 #include "demux_kqueue_demuxer.hpp"
 #include "faio_pdrv.hpp"
 typedef flx::demux::kqueue_demuxer flx_demuxer;
#elif defined(HAVE_EVTPORTS)
 #include "demux_evtport_demuxer.hpp"
 #include "faio_pdrv.hpp"
 typedef flx::demux::evtport_demuxer flx_demuxer;
#elif defined(HAVE_EPOLL)
 #include "demux_epoll_demuxer.hpp"
 #include "faio_pdrv.hpp"
 typedef flx::demux::epoll_demuxer flx_demuxer;
#elif defined(HAVE_POLL)
 // NB!: on osx 10.3 poll exists, but is a poor cousin emulation layer on
 // top of select. however, 10.3 has kqueues (above), so should be ok...
 #include "demux_ts_poll_demuxer.hpp"
 #include "faio_pdrv.hpp"
 typedef flx::demux::ts_poll_demuxer flx_demuxer;
#else
 #include "demux_ts_select_demuxer.hpp"
 #include "faio_pdrv.hpp"
 typedef flx::demux::ts_select_demuxer flx_demuxer;  // thread safe!
#endif

#include "faio_asyncio.hpp"

async_hooker::~async_hooker(){
 //fprintf(stderr,"Deleted async_hooker\n");
}

async_hooker_impl::~async_hooker_impl(){
  //fprintf(stderr,"Deleted async_hooker_impl\n");
}

bool async_hooker_impl::handle_request(void *data,fthread_t *ft)
{
  flx::faio::flx_driver_request_base* dreq =
        (flx::faio::flx_driver_request_base*)data
  ;

  //fprintf(stderr,"Request object at %p\n",dreq);
  // RF hates the flag this function returns .. might
  // mask a race condition, get rid of it

  bool result =dreq->start_async_op(
   *get_demuxer(),
   get_driver(),
  ft)
  ;
  return result;
}

class proto_async : public async_hooker_impl
{
   flx_demuxer demux;

    sleep_queue_t async_active;
    flx_thread_t ethread;

#ifdef _WIN32
    wflx_drv driver;
    static DWORD pthread_thread(LPVOID udat);
#else
    pflx_drv driver;
    static void* pthread_thread(void* udat);
#endif

public:
   proto_async(int n0, int n1, int m1, int n2, int m2) :
     async_active(n0), driver(async_active,n1,m1,n2,m2)
   {
     // start waiting thread. note that we only have ONE event waiting thread.
     // that's currently important for clean takedowns
#if _WIN32
     // cast callback to itself. go figure.
     if(ethread.init((LPTHREAD_START_ROUTINE)pthread_thread, &demux) == -1) throw -1;
#else
     if(ethread.init(pthread_thread, &demux) == -1) throw -1;
#endif
   }

  ~proto_async(){
    //fprintf(stderr,"Deleting proto async\n");
  }
  flx_drv *get_driver() { return &driver; }
  demuxer *get_demuxer(){ return &demux; }
  fthread_t* dequeue()
  {
    return (fthread_t*)async_active.dequeue();
  }
};

#ifdef _WIN32
DWORD
proto_async::pthread_thread(LPVOID udat)
#else
void*
proto_async::pthread_thread(void* udat)
#endif
{
    demuxer*    d = (demuxer*)udat;

    while(1)
    {
        //fprintf(stderr, "ETHREAD ABOUT TO WAIT\n");
        d->wait();          // this does it
        //fprintf(stderr, "ETHREAD CHECKING QUIT FLAG\n");
        demux_quit_flag* f = d->get_quit_flag();
        if(f)
        {
          // got a quit flag - this is the very last thing we do before
          // exiting. don't use the demuxer after this as it's probably been
          // destructed.
          //fprintf(stderr, "ETHREAD GOT QUIT FLAG, SIGNALLING AND EXITING\n");
          f->signal_true();
          // in the case of a system takedown there's no guarantee that
          // anything after the signal_finish will be run at all, so this
          // is not a good place to put anything important.
          break;  // outta here
        }
    }
    //fprintf(stderr, "ETHREAD EXITING\n");
    // fprintf(stderr, "proto_async was asked to quit...\n");
    return 0;
}

async_hooker *create_async_hooker(int n0,int n1,int m1,int n2,int m2) {
  return new proto_async(n0,n1,m1,n2,m2);
}

