#line 233 "./lpsrc/flx_pthread.pak"
#ifndef __FLX_THREAD__
#define __FLX_THREAD__
#include <flx_pthread_config.hpp>

#ifdef _WIN32
#include <windows.h>
#else
#include <pthread.h>
#endif

// auto pthread, because I forget how to deallocate them nicely
// could init in the constructor, but ultimately you don't want the thread
// barging in before you've finished doing other stuff
// Addendum (20051128): doing stdio in turns out to be not very safe.
// I don't know if printf et al are supposed to be thread safe (most impls
// seem to try to be) but I sometimes get deadlocks in ppc64 os x 10.4.2
// with 4.0.1 when printfing to stdout. Nasty.

namespace flx { namespace pthread {

// ********************************************************
/// Posix Threads. This class simply wraps the creation
/// and joining of threads. It is not safe.
// ********************************************************

class PTHREAD_EXTERN flx_detached_thread_t {
#ifdef _WIN32
  HANDLE    thread;
#else
  pthread_t   thr;        ///< the thread
#endif
  flx_detached_thread_t(flx_detached_thread_t const&); // uncopyable
  void operator=(flx_detached_thread_t const&); // uncopyable
public:
  flx_detached_thread_t();
  ~flx_detached_thread_t();
#ifdef _WIN32
  int init(LPTHREAD_START_ROUTINE, LPVOID lParam);
#else
  int init(void* (*start)(void*), void* udat);
#endif
};

class PTHREAD_EXTERN flx_thread_t {
#ifdef _WIN32
  HANDLE    thread;
#else
  pthread_t   thr;        ///< the thread
#endif
  flx_thread_t(flx_thread_t const&); // uncopyable
  void operator=(flx_thread_t const&); // uncopyable
public:
  flx_thread_t();
  ~flx_thread_t();
#ifdef _WIN32
  int init(LPTHREAD_START_ROUTINE, LPVOID lParam);
#else
  int init(void* (*start)(void*), void* udat);
#endif
  void join();
};

/// RAII wrapper for thread class
class PTHREAD_EXTERN flx_thread_wrapper_t {
  flx_thread_t thread;
  flx_thread_wrapper_t(flx_thread_wrapper_t const&); // uncopyable
  void operator=(flx_thread_wrapper_t const&); // uncopyable
public:
  ~flx_thread_wrapper_t();
#ifndef _WIN32
  flx_thread_wrapper_t(void* (*start)(void*), void* udat);
#else
  flx_thread_wrapper_t(LPTHREAD_START_ROUTINE, LPVOID);
#endif
};

}}
#endif

